package com.easefun.polyvsdk;

import android.content.Context;
import android.support.annotation.NonNull;

import com.easefun.polyvsdk.download.listener.IPolyvDownloaderStartListener;
import com.easefun.polyvsdk.download.listener.sdk.IPolyvDownloaderSDKBeforeStartListener;
import com.easefun.polyvsdk.download.listener.sdk.IPolyvDownloaderSDKEndListener;
import com.easefun.polyvsdk.download.listener.sdk.IPolyvDownloaderSDKStopListener;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * Polyv 下载器管理
 * @author Lion 2015-09-09
 */
public class PolyvDownloaderManager {
	private static final LinkedHashMap<String, PolyvDownloader> polyvDownloaderMap = new LinkedHashMap<>();
	private static final ArrayList<String> downloadingList = new ArrayList<>();
	private static final ArrayList<String> waitingList = new ArrayList<>();
	/** 下载队列总数 */
	private static int downloadQueueCount = 1;

	private PolyvDownloaderManager() {

	}

	/**
	 * 取得{@link PolyvDownloader}，没有缓存的{@link PolyvDownloader}则新建一个，并加入缓存中。<br/>
	 * 如果视频开始下载就会回调{@link PolyvDownloader#setPolyvDownloadStartListener(IPolyvDownloaderStartListener)}<br/>
	 * 如果调用了{@link PolyvDownloader#start()}方法或者调用了{@link PolyvDownloader#start(Context)}方法，而没有回调{@link PolyvDownloader#setPolyvDownloadStartListener(IPolyvDownloaderStartListener)}就表示正在排队中。<br/>
	 * 可以调用{@link #isWaitingDownload(String, int)}方法来获取视频是否在排队中。
	 * @param vid 视频id
	 * @param bitRate 码率（清晰度）
	 * @return 下载器
	 */
	@NonNull
	public static PolyvDownloader getPolyvDownloader(@NonNull String vid, int bitRate) {
		return getPolyvDownloader(vid, bitRate, Video.HlsSpeedType.SPEED_1X);
	}

	/**
	 * 取得{@link PolyvDownloader}，没有缓存的{@link PolyvDownloader}则新建一个，并加入缓存中。<br/>
	 * 如果视频开始下载就会回调{@link PolyvDownloader#setPolyvDownloadStartListener(IPolyvDownloaderStartListener)}<br/>
	 * 如果调用了{@link PolyvDownloader#start()}方法或者调用了{@link PolyvDownloader#start(Context)}方法，而没有回调{@link PolyvDownloader#setPolyvDownloadStartListener(IPolyvDownloaderStartListener)}就表示正在排队中。<br/>
	 * 可以调用{@link #isWaitingDownload(String, int)}方法来获取视频是否在排队中。
	 * @param vid 视频id
	 * @param bitRate 码率（清晰度）
	 * @param hlsSpeedType 播放速度，该参数已废弃
	 * @return 下载器
	 * @see #getPolyvDownloader(String, int)
	 */
	@NonNull
	@Deprecated
	public static PolyvDownloader getPolyvDownloader(@NonNull final String vid, final int bitRate, @NonNull Video.HlsSpeedType hlsSpeedType) {
		String key = getKey(vid, bitRate, hlsSpeedType);
		synchronized (PolyvDownloaderManager.class) {
			PolyvDownloader polyvDownloader = polyvDownloaderMap.get(key);
			if (polyvDownloader == null) {
				polyvDownloader = new PolyvDownloader(vid, bitRate, hlsSpeedType);
				polyvDownloader.setPolyvDownloadSDKBeforeStartListener(new IPolyvDownloaderSDKBeforeStartListener() {
					@Override
					public boolean onBeforeStart() {
						synchronized (PolyvDownloaderManager.class) {
							String key = getKey(vid, bitRate);
							if (downloadQueueCount == 0 || downloadingList.size() < downloadQueueCount) {
								downloadingList.add(key);
								waitingList.remove(key);
								return true;
							}

							if (!waitingList.contains(key)) {
								waitingList.add(key);
							}

							return false;
						}
					}
				});

				polyvDownloader.setPolyvDownloadSDKStopListener(new IPolyvDownloaderSDKStopListener() {
					@Override
					public void onStop() {
						updateDownloadQueue(vid, bitRate);
					}
				});

				polyvDownloader.setPolyvDownloadSDKEndListener(new IPolyvDownloaderSDKEndListener() {
					@Override
					public void onSuccess() {
						updateDownloadQueue(vid, bitRate);
					}

					@Override
					public void onFail() {
						updateDownloadQueue(vid, bitRate);
					}
				});

				polyvDownloaderMap.put(key, polyvDownloader);
			}

			return polyvDownloader;
		}
	}

	private static void updateDownloadQueue(String vid, int bitRate) {
		synchronized (PolyvDownloaderManager.class) {
			String key = getKey(vid, bitRate);
			if (downloadingList.contains(key)) {
				downloadingList.remove(key);
				if (downloadQueueCount == 0 || downloadingList.size() < downloadQueueCount) {
					if (!waitingList.isEmpty()) {
						PolyvDownloader polyvDownloader = null;
						for (String nextKey : waitingList) {
							if (polyvDownloaderMap.containsKey(nextKey)) {
								polyvDownloader = polyvDownloaderMap.get(nextKey);
								break;
							}
						}

						if (polyvDownloader == null) {
							waitingList.clear();
						} else {
							polyvDownloader.start(polyvDownloader.getContext());
						}
					}
				}
			} else if (waitingList.contains(key)) {
				waitingList.remove(key);
			}
		}
	}

	/**
	 * 是否在等待下载中
	 * @param vid 视频id
	 * @param bitRate 码率（清晰度）
	 * @return {@code true}:在等待下载中<br/>{@code false}:不在等待下载中
	 */
	public static boolean isWaitingDownload(String vid, int bitRate) {
		String key = getKey(vid, bitRate);
		return waitingList.contains(key);
	}

	/**
	 * 清除{@link PolyvDownloader}<br/>
	 * 会删除{@link PolyvDownloader}的监听回调，会调用{@link PolyvDownloader#stop(boolean)}方法，参数传递为{@code true}，并删除缓存。
	 * @param vid 视频id
	 * @param bitRate 码率（清晰度）
	 * @return 下载器
	 */
	@NonNull
	public static PolyvDownloader clearPolyvDownload(@NonNull String vid, int bitRate) {
		return clearPolyvDownload(vid, bitRate, Video.HlsSpeedType.SPEED_1X);
	}
	
	/**
	 * 清除{@link PolyvDownloader}<br/>
	 * 会删除{@link PolyvDownloader}的监听回调，会调用{@link PolyvDownloader#stop(boolean)}方法，参数传递为{@code true}，并删除缓存。
	 * @param vid 视频id
	 * @param bitRate 码率（清晰度）
	 * @param hlsSpeedType 播放速度，该参数已废弃
	 * @return 下载器
	 * @see #clearPolyvDownload(String, int)
	 */
	@NonNull
	@Deprecated
	public static PolyvDownloader clearPolyvDownload(@NonNull String vid, int bitRate, @NonNull Video.HlsSpeedType hlsSpeedType) {
		PolyvDownloader polyvDownloader = getPolyvDownloader(vid, bitRate, hlsSpeedType);
		polyvDownloader.stop(true);
		synchronized (PolyvDownloaderManager.class) {
			polyvDownloaderMap.remove(getKey(vid, bitRate, hlsSpeedType));
			return polyvDownloader;
		}
	}

	/**
	 * 开始全部下载
	 * @see #startAll(Context)
	 */
	@Deprecated
	public static void startAll() {
		startAll(null);
	}

	/**
	 * 开始全部下载
	 * @param context 设置此参数会判断SDK卡权限是否拥有
	 */
	public static void startAll(Context context) {
		for (Map.Entry<String, PolyvDownloader> entry : polyvDownloaderMap.entrySet()) {
			entry.getValue().start(context);
		}
	}

	/**
	 * 获取key
	 * @param vid 视频id
	 * @param bitRate 码率（清晰度）
	 * @return key
	 */
	@NonNull
	public static String getKey(@NonNull String vid, int bitRate) {
		return getKey(vid, bitRate, Video.HlsSpeedType.SPEED_1X);
	}

	/**
	 * 获取key
	 * @param vid 视频id
	 * @param bitRate 码率（清晰度）
	 * @param hlsSpeedType 播放速度，该参数已废弃
	 * @return key
	 * @see #getKey(String, int)
	 */
	@NonNull
	@Deprecated
	public static String getKey(@NonNull String vid, int bitRate, @NonNull Video.HlsSpeedType hlsSpeedType) {
		return vid + "_" + bitRate + "_" + hlsSpeedType.getName();
	}

	/**
	 * 开始未完成的任务 <br/>
	 * 获取key见方法{@link #getKey(String, int)}
	 * @param finishKeys 已完成任务的key集合
	 * @see #startUnfinished(List, Context)
	 */
	@Deprecated
	public static void startUnfinished(List<String> finishKeys) {
		startUnfinished(finishKeys, null);
	}

	/**
	 * 开始未完成的任务 <br/>
	 * 获取key见方法{@link #getKey(String, int)}
	 * @param finishKeys 已完成任务的key集合
	 * @param context The context to use
	 */
	public static void startUnfinished(List<String> finishKeys, Context context) {
		for (Map.Entry<String, PolyvDownloader> entry : polyvDownloaderMap.entrySet()) {
			if (finishKeys != null && !finishKeys.isEmpty() && finishKeys.contains(entry.getKey())) {
				continue;
			}

			entry.getValue().start(context);
		}
	}

	/**
	 * 停止全部下载
	 */
	public static void stopAll() {
		synchronized (PolyvDownloaderManager.class) {
			//删除所有等待的下载
			waitingList.clear();
		}

		for (Map.Entry<String, PolyvDownloader> entry : polyvDownloaderMap.entrySet()) {
			entry.getValue().stop();
		}
	}

	/**
	 * 设置下载队列总数，多少个视频能同时下载。
	 * @param count 总数，默认是1，设置负数和0是没有限制
	 */
	public static void setDownloadQueueCount(int count) {
		if (count < 0) {
			count = 0;
		}

		downloadQueueCount = count;
	}

	/**
	 * 获取下载队列总数
	 * @return 队列总数
	 */
	public static int getDownloadQueueCount() {
		return downloadQueueCount;
	}
}
